@@ -24,7 +24,7 @@ import contextlib
 
 from dataiter import util
 
-__version__ = "0.29.2"
+__version__ = "0.35"
 
 DEFAULT_PEEK_ELEMENTS = 10
 DEFAULT_PEEK_ITEMS = 3
@@ -34,29 +34,37 @@ PRINT_MAX_ELEMENTS = 100
 PRINT_MAX_ITEMS = 10
 PRINT_MAX_ROWS = 100
 PRINT_MAX_WIDTH = 80
+PRINT_TRUNCATE_WIDTH = 36
 USE_NUMBA = False
 
-with contextlib.suppress(Exception):
-    # Use Numba automatically if found.
-    import numba
-    USE_NUMBA = True
-    del numba
-
-with contextlib.suppress(LookupError):
-    # Force Numba on or off via an environment variable.
+try:
+    # Force Numba on or off if environment variable defined.
     USE_NUMBA = util.parse_env_boolean("DATAITER_USE_NUMBA")
+except LookupError:
+    with contextlib.suppress(Exception):
+        # Use Numba automatically if found
+        # and a calling a trivial function works.
+        import numba
+        try:
+            @numba.njit(cache=True)
+            def check(x):
+                return x**2
+            assert check(10) == 100
+            USE_NUMBA = True
+        except Exception as error:
+            print(f"Numba found, but disabled due to error: {error!s}")
 
-del contextlib
-del util
+globals().pop("check", None)
+globals().pop("contextlib", None)
+globals().pop("numba", None)
+globals().pop("util", None)
 
-# Classes API
 from dataiter.vector import Vector # noqa
 from dataiter.data_frame import DataFrame # noqa
 from dataiter.data_frame import DataFrameColumn # noqa
 from dataiter.geojson import GeoJSON # noqa
 from dataiter.list_of_dicts import ListOfDicts # noqa
 
-# Functions API
 from dataiter.aggregate import all # noqa
 from dataiter.aggregate import any # noqa
 from dataiter.aggregate import count # noqa
@@ -74,16 +82,8 @@ from dataiter.aggregate import quantile # noqa
 from dataiter.aggregate import std # noqa
 from dataiter.aggregate import sum # noqa
 from dataiter.aggregate import var # noqa
+
 from dataiter.io import read_csv # noqa
 from dataiter.io import read_geojson # noqa
 from dataiter.io import read_json # noqa
 from dataiter.io import read_npz # noqa
-
-for name, value in list(globals().items()):
-    if (getattr(value, "__module__", "") in
-        ["dataiter.aggregate", "dataiter.io"]):
-        # Patch module to include value in API documentation.
-        value.__module__ = "dataiter"
-
-del name
-del value
