@@ -30,6 +30,7 @@ import pickle
 from dataiter import deco
 from dataiter import util
 from dataiter import Vector
+from math import inf
 
 
 class DataFrameColumn(Vector):
@@ -198,14 +199,14 @@ class DataFrame(dict):
         data._index_ = np.arange(data.nrow)
         stat = data.unique(*group_colnames).select("_index_", *group_colnames)
         indices = np.split(data._index_, stat._index_[1:])
-        uses_numba = [getattr(x, "numba", False) for x in colname_function_pairs.values()]
-        if any(uses_numba):
+        group_aware = [getattr(x, "group_aware", False) for x in colname_function_pairs.values()]
+        if any(group_aware):
             groups = Vector.fast(range(len(indices)), int)
             n = Vector.fast(map(len, indices), int)
             data._group_ = np.repeat(groups, n)
         slices = None
         for colname, function in colname_function_pairs.items():
-            if getattr(function, "numba", False):
+            if getattr(function, "group_aware", False):
                 # function might leave Nones in its output,
                 # once those are replaced with the proper default
                 # we can do a fast conversion to DataFrameColumn.
@@ -240,7 +241,7 @@ class DataFrame(dict):
         >>> reviews = di.read_csv("data/listings-reviews.csv")
         >>> listings.anti_join(reviews, "id")
         """
-        by1, by2 = self._split_by(*by)
+        by1, by2 = self._split_join_by(*by)
         other = other.unique(*by2)
         found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
@@ -294,6 +295,67 @@ class DataFrame(dict):
         """
         return list(self.values())
 
+    def compare(self, other, *by, ignore_columns=[], max_changed=inf):
+        """
+        Find differences against another data frame.
+
+        `by` are identifier columns which are used to uniquely identify rows
+        and match them between `self` and `other`. `compare` will not work if
+        your data lacks suitable identifiers. `ignore_columns` is an optional
+        list of columns, differences in which to ignore.
+
+        `compare` returns three data frames: added rows, removed rows and
+        changed values. The first two are basically subsets of the rows of
+        `self` and `other`, respectively. Changed values are returned as a data
+        frame with one row per differing value (not per differing row). Listing
+        changes will terminate once `max_changed` is reached.
+
+        .. warning:: `compare` is experimental, do not rely on it reporting all
+                     of the differences correctly. Do not try to give it two
+                     huge data frames with very little in common, unless also
+                     giving some sensible value for `max_changed`.
+
+        >>> old = di.read_csv("data/vehicles.csv")
+        >>> new = old.modify(hwy=lambda x: np.minimum(100, x.hwy))
+        >>> added, removed, changed = new.compare(old, "id")
+        >>> changed
+        """
+        if self.unique(*by).nrow < self.nrow:
+            raise ValueError(f"self not unique by {by}")
+        if other.unique(*by).nrow < other.nrow:
+            raise ValueError(f"other not unique by {by}")
+        added = self.anti_join(other, *by)
+        removed = other.anti_join(self, *by)
+        x = self.modify(_i_=range(self.nrow))
+        y = other.modify(_j_=range(other.nrow))
+        z = x.inner_join(y.select("_j_", *by), *by)
+        colnames = util.unique_keys(self.colnames + other.colnames)
+        colnames = [x for x in colnames if x not in ignore_columns]
+        changed = []
+        for i, j in zip(z._i_, z._j_):
+            if len(changed) >= max_changed:
+                print(f"max_changed={max_changed} reached, terminating")
+                break
+            for colname in colnames:
+                if len(changed) >= max_changed: break
+                # XXX: How to make a distinction between
+                # a missing column and a missing value?
+                xvalue = x[colname][i] if colname in x else None
+                yvalue = y[colname][j] if colname in y else None
+                if (xvalue != yvalue and
+                    not Vector([xvalue, yvalue]).is_na().all()):
+                    # XXX: We could have a name clash here.
+                    byrow = {k: x[k][i] for k in by}
+                    changed.append(dict(**byrow,
+                                        column=colname,
+                                        xvalue=xvalue,
+                                        yvalue=yvalue))
+
+        added = added if added.nrow > 0 else None
+        removed = removed if removed.nrow > 0 else None
+        changed = self.from_json(changed) if changed else None
+        return added, removed, changed
+
     def copy(self):
         """
         Return a shallow copy.
@@ -365,7 +427,7 @@ class DataFrame(dict):
             yield colname, np.delete(column, rows)
 
     @classmethod
-    def from_json(cls, string, columns=[], dtypes={}, **kwargs):
+    def from_json(cls, string, *, columns=[], dtypes={}, **kwargs):
         """
         Return a new data frame from JSON `string`.
 
@@ -373,7 +435,9 @@ class DataFrame(dict):
         optional dict mapping column names to NumPy datatypes. `kwargs` are
         passed to ``json.load``.
         """
-        data = json.loads(string, **kwargs)
+        data = string
+        if isinstance(data, str):
+            data = json.loads(data, **kwargs)
         if not isinstance(data, list):
             raise TypeError("Not a list")
         keys = util.unique_keys(itertools.chain(*data))
@@ -385,15 +449,18 @@ class DataFrame(dict):
         return cls(**data)
 
     @classmethod
-    def from_pandas(cls, data, dtypes={}):
+    def from_pandas(cls, data, *, dtypes={}):
         """
         Return a new data frame from ``pandas.DataFrame`` `data`.
 
         `dtypes` is an optional dict mapping column names to NumPy datatypes.
         """
-        # It would be a lot faster to skip tolist here,
-        # but then we'd need to map some Pandas dtype oddities.
-        data = {x: data[x].to_numpy().tolist() for x in data.columns}
+        data = {x: data[x].to_numpy(copy=True) for x in data.columns}
+        for name, value in data.items():
+            # Pandas object columns are likely to be strings,
+            # convert to list to force type guessing in Vector.__init__.
+            if np.issubdtype(value.dtype, np.object_):
+                data[name] = data[name].tolist()
         for name, dtype in dtypes.items():
             data[name] = DataFrameColumn(data[name], dtype)
         return cls(**data)
@@ -422,7 +489,16 @@ class DataFrame(dict):
         b = b.anti_join(ab, "_bid_")
         if b.nrow == 0:
             return ab.unselect("_aid_", "_bid_")
-        ba = b.left_join(a, *by)
+        by_reverse = [
+            tuple(reversed(x)) if isinstance(x, (list, tuple))
+            else x
+            for x in by]
+        ba = b.left_join(a, *by_reverse)
+        for item in by:
+            # For identifiers in by whose name differs in a and b,
+            # rename and keep the variant found in a.
+            if isinstance(item, (list, tuple)):
+                ba[item[0]] = ba.pop(item[1])
         return ab.rbind(ba).sort(_aid_=1, _bid_=1).unselect("_aid_", "_bid_")
 
     def _get_join_indices(self, other, by1, by2):
@@ -470,7 +546,7 @@ class DataFrame(dict):
         >>> reviews = di.read_csv("data/listings-reviews.csv")
         >>> listings.inner_join(reviews, "id")
         """
-        by1, by2 = self._split_by(*by)
+        by1, by2 = self._split_join_by(*by)
         other = other.unique(*by2)
         found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
@@ -497,7 +573,7 @@ class DataFrame(dict):
         >>> reviews = di.read_csv("data/listings-reviews.csv")
         >>> listings.left_join(reviews, "id")
         """
-        by1, by2 = self._split_by(*by)
+        by1, by2 = self._split_join_by(*by)
         other = other.unique(*by2)
         found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
@@ -505,8 +581,8 @@ class DataFrame(dict):
         for colname, column in other.items():
             if colname in by2: continue
             if colname in self: continue
-            value = column.missing_value
-            dtype = column.missing_dtype
+            value = column.na_value
+            dtype = column.na_dtype
             new = DataFrameColumn(value, dtype, self.nrow)
             new[found] = column[src[found]]
             yield colname, new.copy()
@@ -524,7 +600,7 @@ class DataFrame(dict):
         vectorized form.
 
         >>> data = di.read_csv("data/listings-reviews.csv")
-        >>> data.map(lambda x, i: min(1, x.reviews[i]/100) * min(1, x.rating[i]/100))
+        >>> data.map(lambda x, i: (x.reviews[i], x.rating[i]))
         """
         return [function(self, i) for i in range(self.nrow)]
 
@@ -545,12 +621,28 @@ class DataFrame(dict):
         >>> data = di.read_csv("data/listings.csv")
         >>> data.modify(price_per_guest=data.price/data.guests)
         >>> data.modify(price_per_guest=lambda x: x.price / x.guests)
+
+        If the data frame is grouped, then `colname_value_pairs` need to be
+        functions, which are applied to group-wise subsets of the data frame.
+        A common use for this is calculating group-wise fractions.
+
+        >>> data = di.DataFrame(g=[1, 2, 2, 3, 3, 3])
+        >>> data.group_by("g").modify(f=lambda x: 1 / x.nrow)
         """
         for colname, column in self.items():
             yield colname, column.copy()
-        for colname, value in colname_value_pairs.items():
-            value = value(self) if callable(value) else value
-            yield colname, self._reconcile_column(value).copy()
+        if self._group_colnames:
+            slices = self.split(*self._group_colnames)
+            slices = [self._view_rows(x) for x in slices]
+            for colname, function in colname_value_pairs.items():
+                if not callable(function):
+                    raise ValueError(f"{colname} argument not callable")
+                column = [DataFrameColumn(function(x), nrow=x.nrow) for x in slices]
+                yield colname, np.concatenate(column)
+        else:
+            for colname, value in colname_value_pairs.items():
+                value = value(self) if callable(value) else value
+                yield colname, self._reconcile_column(value).copy()
 
     @property
     def ncol(self):
@@ -596,7 +688,7 @@ class DataFrame(dict):
     def _parse_rows_from_integer(self, rows):
         return Vector.fast(rows, int)
 
-    def print_(self, max_rows=None, max_width=None):
+    def print_(self, *, max_rows=None, max_width=None, truncate_width=None):
         """
         Print data frame to ``sys.stdout``.
 
@@ -606,7 +698,9 @@ class DataFrame(dict):
 
         >>> di.read_csv("data/listings.csv").print_()
         """
-        print(self.to_string(max_rows, max_width))
+        print(self.to_string(max_rows=max_rows,
+                             max_width=max_width,
+                             truncate_width=truncate_width))
 
     def print_memory_use(self):
         """
@@ -633,16 +727,16 @@ class DataFrame(dict):
         mem.colnames = [x.upper() for x in mem.colnames]
         print(mem)
 
-    def print_missing_counts(self):
+    def print_na_counts(self):
         """
         Print counts of missing values by column.
 
         >>> data = di.read_csv("data/listings.csv")
-        >>> data.print_missing_counts()
+        >>> data.print_na_counts()
         """
         nas = DataFrame()
         for name in self.colnames:
-            n = self[name].is_missing().sum()
+            n = self[name].is_na().sum()
             if n == 0: continue
             nas = nas.rbind(DataFrame(column=name, nna=n))
         if not nas: return
@@ -665,8 +759,8 @@ class DataFrame(dict):
                 return data[colname]
             for ref in data_frames:
                 if colname not in ref: continue
-                value = ref[colname].missing_value
-                dtype = ref[colname].missing_dtype
+                value = ref[colname].na_value
+                dtype = ref[colname].na_dtype
                 return Vector.fast([value], dtype).repeat(data.nrow)
         for colname in colnames:
             parts = [get_part(x, colname) for x in data_frames]
@@ -674,7 +768,7 @@ class DataFrame(dict):
             yield colname, total
 
     @classmethod
-    def read_csv(cls, path, encoding="utf-8", sep=",", header=True, columns=[], dtypes={}):
+    def read_csv(cls, path, *, encoding="utf-8", sep=",", header=True, columns=[], dtypes={}):
         """
         Return a new data frame from CSV file `path`.
 
@@ -695,10 +789,10 @@ class DataFrame(dict):
 
         if not header:
             data.columns = util.generate_colnames(len(data.columns))
-        return cls.from_pandas(data, dtypes)
+        return cls.from_pandas(data, dtypes=dtypes)
 
     @classmethod
-    def read_json(cls, path, encoding="utf-8", columns=[], dtypes={}, **kwargs):
+    def read_json(cls, path, *, encoding="utf-8", columns=[], dtypes={}, **kwargs):
         """
         Return a new data frame from JSON file `path`.
 
@@ -709,10 +803,10 @@ class DataFrame(dict):
         passed to ``json.load``.
         """
         with util.xopen(path, "rt", encoding=encoding) as f:
-            return cls.from_json(f.read(), columns, dtypes, **kwargs)
+            return cls.from_json(f.read(), columns=columns, dtypes=dtypes, **kwargs)
 
     @classmethod
-    def read_npz(cls, path, allow_pickle=True):
+    def read_npz(cls, path, *, allow_pickle=True):
         """
         Return a new data frame from NumPy file `path`.
 
@@ -790,7 +884,7 @@ class DataFrame(dict):
         >>> reviews = di.read_csv("data/listings-reviews.csv")
         >>> listings.semi_join(reviews, "id")
         """
-        by1, by2 = self._split_by(*by)
+        by1, by2 = self._split_join_by(*by)
         other = other.unique(*by2)
         found, src = self._get_join_indices(other, by1, by2)
         for colname, column in self.items():
@@ -834,6 +928,9 @@ class DataFrame(dict):
                 if dir not in [1, -1]:
                     raise ValueError("dir should be 1 or -1")
                 column = self[colname]
+                if column.is_object():
+                    # Avoid TypeError trying to compare different types.
+                    column = column.as_string()
                 if dir < 0 and not (column.is_boolean() or column.is_number()):
                     # Use rank for non-numeric types so that we can sort descending.
                     column = column.rank(method="min")
@@ -842,7 +939,21 @@ class DataFrame(dict):
         for colname, column in self.items():
             yield colname, column[indices].copy()
 
-    def _split_by(self, *by):
+    def split(self, *by):
+        """
+        Split data frame into groups and return a list of their rows.
+
+        >>> data = di.DataFrame(x=[1, 2, 2, 3, 3, 3])
+        >>> data.split("x")
+        """
+        data = self.select(*by)
+        data._index_ = np.arange(data.nrow)
+        data = data.sort(**dict.fromkeys(by, 1))
+        data._sorted_index_ = np.arange(data.nrow)
+        stat = data.unique(*by)
+        return np.split(data._index_, stat._sorted_index_[1:])
+
+    def _split_join_by(self, *by):
         by1 = [x if isinstance(x, str) else x[0] for x in by]
         by2 = [x if isinstance(x, str) else x[1] for x in by]
         return by1, by2
@@ -894,7 +1005,7 @@ class DataFrame(dict):
         import pandas as pd
         return pd.DataFrame({x: self[x].tolist() for x in self.colnames})
 
-    def to_string(self, max_rows=None, max_width=None):
+    def to_string(self, *, max_rows=None, max_width=None, truncate_width=None):
         """
         Return data frame as a string formatted for display.
 
@@ -902,16 +1013,18 @@ class DataFrame(dict):
         >>> data.to_string()
         """
         if not self: return ""
-        max_rows = dataiter.PRINT_MAX_ROWS if max_rows is None else max_rows
-        max_width = util.get_print_width() if max_width is None else max_width
+        max_rows = max_rows or dataiter.PRINT_MAX_ROWS
+        max_width = max_width or util.get_print_width()
+        truncate_width = truncate_width or dataiter.PRINT_TRUNCATE_WIDTH
         n = min(self.nrow, max_rows)
         columns = {colname: util.pad(
             [colname] +
             [str(column.dtype)] +
-            [str(x) for x in column[:n].to_strings(quote=False, pad=True)]
+            [str(x) for x in column[:n].to_strings(
+                quote=False, pad=True, truncate_width=truncate_width)]
         ) for colname, column in self.items()}
         for column in columns.values():
-            column.insert(2, "-" * len(column[0]))
+            column.insert(2, "─" * util.ulen(column[0]))
         row_numbers = [str(i) for i in range(n)]
         row_numbers = util.pad(["", "", ""] + row_numbers)
         # If the length of rows exceeds max_width, split to
@@ -922,7 +1035,7 @@ class DataFrame(dict):
             batch_rows = [" ".join(x) for x in zip(
                 row_numbers, columns.pop(first))]
             for colname, column in list(columns.items()):
-                width = len(batch_rows[0] + column[0]) + 1
+                width = util.ulen(batch_rows[0] + column[0]) + 1
                 if width > max_width: break
                 for i in range(len(column)):
                     batch_rows[i] += " "
@@ -993,7 +1106,7 @@ class DataFrame(dict):
         dict.update(data, {x: self[x][rows] for x in self})
         return data
 
-    def write_csv(self, path, encoding="utf-8", header=True, sep=","):
+    def write_csv(self, path, *, encoding="utf-8", header=True, sep=","):
         """
         Write data frame to CSV file `path`.
 
@@ -1003,7 +1116,7 @@ class DataFrame(dict):
         util.makedirs_for_file(path)
         pddf.to_csv(path, sep=sep, header=header, index=False, encoding=encoding)
 
-    def write_json(self, path, encoding="utf-8", **kwargs):
+    def write_json(self, path, *, encoding="utf-8", **kwargs):
         """
         Write data frame to JSON file `path`.
 
@@ -1013,7 +1126,7 @@ class DataFrame(dict):
         """
         return self.to_list_of_dicts().write_json(path, encoding=encoding, **kwargs)
 
-    def write_npz(self, path, compress=False):
+    def write_npz(self, path, *, compress=False):
         """
         Write data frame to NumPy file `path`.
         """
