@@ -26,6 +26,7 @@ import numpy as np
 import sys
 
 from dataiter import util
+from math import inf
 
 TYPE_CONVERSIONS = {
     datetime.date: "datetime64[D]",
@@ -97,6 +98,9 @@ class Vector(np.ndarray):
         >>> vector.as_boolean()
         """
         if self.is_string():
+            # NumPy does bool(int(str)), which is weird.
+            # https://github.com/numpy/numpy/issues/20898
+            # https://github.com/numpy/numpy/pull/21024
             return self.map(bool)
         return self.astype(bool)
 
@@ -157,27 +161,28 @@ class Vector(np.ndarray):
         """
         return self.__class__(self.tolist(), object)
 
-    def as_string(self):
+    def as_string(self, length=None):
         """
         Return vector converted to string data type.
 
         >>> vector = di.Vector([1, 2, 3])
         >>> vector.as_string()
+        >>> vector.as_string(64)
         """
-        return self.astype(str)
+        return self.astype(f"U{length}" if length else str)
 
     def _check_dimensions(self):
         if self.ndim == 1: return
         raise ValueError(f"Bad dimensions: {self.ndim!r}")
 
-    def drop_missing(self):
+    def drop_na(self):
         """
         Return vector without missing values.
 
         >>> vector = di.Vector([1, 2, 3, None])
-        >>> vector.drop_missing()
+        >>> vector.drop_na()
         """
-        return self[~self.is_missing()].copy()
+        return self[~self.is_na()].copy()
 
     def equal(self, other):
         """
@@ -194,10 +199,10 @@ class Vector(np.ndarray):
         """
         if not (isinstance(other, Vector) and
                 self.length == other.length and
-                str(self.missing_value) == str(other.missing_value)):
+                str(self.na_value) == str(other.na_value)):
             return False
-        ii = self.is_missing()
-        jj = other.is_missing()
+        ii = self.is_na()
+        jj = other.is_na()
         return (np.all(ii == jj) and
                 np.all(self[~ii] == other[~jj]))
 
@@ -271,16 +276,18 @@ class Vector(np.ndarray):
         """
         return np.issubdtype(self.dtype, np.integer)
 
-    def is_missing(self):
+    def is_na(self):
         """
         Return a boolean vector indicating missing data elements.
 
         >>> vector = di.Vector([1, 2, 3, None])
         >>> vector
-        >>> vector.is_missing()
+        >>> vector.is_na()
         """
         if self.is_datetime():
             return np.isnat(self)
+        if self.is_timedelta():
+            return np.isnat(self)
         if self.is_float():
             return np.isnan(self)
         if self.is_string():
@@ -305,6 +312,12 @@ class Vector(np.ndarray):
         """
         return np.issubdtype(self.dtype, np.unicode_)
 
+    def is_timedelta(self):
+        """
+        Return whether vector data type is timedelta.
+        """
+        return np.issubdtype(self.dtype, np.timedelta64)
+
     @property
     def length(self):
         """
@@ -327,7 +340,7 @@ class Vector(np.ndarray):
         return self.__class__(function(x, *args, **kwargs) for x in self)
 
     @property
-    def missing_dtype(self):
+    def na_dtype(self):
         """
         Return the corresponding data type that can handle missing data.
 
@@ -335,14 +348,16 @@ class Vector(np.ndarray):
 
         >>> vector = di.Vector([1, 2, 3])
         >>> vector
-        >>> vector.put([2], vector.missing_value)
-        >>> vector = vector.astype(vector.missing_dtype)
+        >>> vector.put([2], vector.na_value)
+        >>> vector = vector.astype(vector.na_dtype)
         >>> vector
-        >>> vector.put([2], vector.missing_value)
+        >>> vector.put([2], vector.na_value)
         >>> vector
         """
         if self.is_datetime():
             return self.dtype
+        if self.is_timedelta():
+            return self.dtype
         if self.is_float():
             return self.dtype
         if self.is_integer():
@@ -352,29 +367,38 @@ class Vector(np.ndarray):
         return object
 
     @property
-    def missing_value(self):
+    def na_value(self):
         """
         Return the corresponding value to use to represent missing data.
 
         Dataiter is built on top of NumPy. NumPy doesn't support a proper
-        missing value ("NA"), only two data type specific values: ``np.nan``
-        and ``np.datetime64("NaT")``. Dataiter recommends the following values
-        be used and internally supports them to an extent.
+        missing value ("NA"), only data type specific values: ``np.nan``,
+        ``np.datetime64("NaT")`` and ``np.timedelta64("NaT")``. Dataiter
+        recommends the following values be used and internally supports them to
+        an extent.
 
-        ======== ========================
-        datetime ``np.datetime64("NaT")``
-        float    ``np.nan``
-        integer  ``np.nan``
-        string   ``""``
-        other    ``None``
-        ======== ========================
+        ========= ========================
+        datetime  ``np.datetime64("NaT")``
+        float     ``np.nan``
+        integer   ``np.nan``
+        string    ``""``
+        timedelta ``np.timedelta64("NaT")``
+        other     ``None``
+        ========= ========================
 
         Note that actually using these might require upcasting the vector.
         Integer will need to be upcast to float to contain ``np.nan``. Other,
         such as boolean, will need to be upcast to object to contain ``None``.
+
+        If you need to avoid object columns, you can also consider converting
+        booleans to float using :meth:`as_float`, which will give you 0.0 for
+        false and 1.0 for true. Depending on how you use the data, that might
+        work as well as an object vector of ``True``, ``False`` and ``None``.
         """
         if self.is_datetime():
             return np.datetime64("NaT")
+        if self.is_timedelta():
+            return np.timedelta64("NaT")
         if self.is_float():
             return np.nan
         if self.is_integer():
@@ -395,7 +419,7 @@ class Vector(np.ndarray):
         rng = [np.nanmin(self), np.nanmax(self)]
         return self.__class__(rng, self.dtype)
 
-    def rank(self, method="average"):
+    def rank(self, *, method="average"):
         """
         Return the order of elements in a sorted vector.
 
@@ -421,7 +445,7 @@ class Vector(np.ndarray):
         """
         if method not in ["min", "max", "average", "ordinal"]:
             raise ValueError(f"Unexpected method: {method!r}")
-        missing = self.is_missing()
+        na = self.is_na()
         if method == "average":
             rank_min = self.rank(method="min")
             rank_max = self.rank(method="max")
@@ -429,34 +453,34 @@ class Vector(np.ndarray):
             return self.__class__(rank)
         if method == "min":
             # https://stackoverflow.com/a/14672797/16369038
-            inv = np.unique(self[~missing], return_inverse=True)[1]
+            inv = np.unique(self[~na], return_inverse=True)[1]
             arank = np.concatenate(([0], np.bincount(inv))).cumsum()[inv]
             zrank = arank.max() + 1
         if method == "max":
             # https://stackoverflow.com/a/14672797/16369038
-            inv = np.unique(self[~missing], return_inverse=True)[1]
+            inv = np.unique(self[~na], return_inverse=True)[1]
             arank = np.bincount(inv).cumsum()[inv] - 1
             zrank = len(self) - 1
         if method == "ordinal":
             # https://stackoverflow.com/a/5284703/16369038
-            indices = self[~missing].argsort()
+            indices = self[~na].argsort()
             arank = np.empty_like(indices)
             arank[indices] = np.arange(len(indices))
-            zrank = arank.max() + 1 + np.arange(missing.sum())
+            zrank = arank.max() + 1 + np.arange(na.sum())
         out = np.zeros_like(self, int)
-        out[~missing] = arank + 1
-        out[missing] = zrank + 1
+        out[~na] = arank + 1
+        out[na] = zrank + 1
         return self.__class__(out)
 
-    def replace_missing(self, value):
+    def replace_na(self, value):
         """
         Return vector with missing values replaced with `value`.
 
         >>> vector = di.Vector([1, 2, 3, None])
-        >>> vector.replace_missing(0)
+        >>> vector.replace_na(0)
         """
         vector = self.copy()
-        vector[vector.is_missing()] = value
+        vector[vector.is_na()] = value
         return vector
 
     def sample(self, n=None):
@@ -472,7 +496,7 @@ class Vector(np.ndarray):
         indices = np.random.choice(self.length, n, replace=False)
         return self[np.sort(indices)].copy()
 
-    def sort(self, dir=1):
+    def sort(self, *, dir=1):
         """
         Return elements in sorted order.
 
@@ -484,9 +508,12 @@ class Vector(np.ndarray):
         >>> vector.sort(dir=1)
         >>> vector.sort(dir=-1)
         """
-        missing = self.is_missing()
-        a = self[~missing].copy()
-        z = self[missing].copy()
+        if self.is_object():
+            # Avoid TypeError trying to compare different types.
+            return self.as_string().sort(dir=dir)
+        na = self.is_na()
+        a = self[~na].copy()
+        z = self[na].copy()
         np.ndarray.sort(a)
         if dir < 0:
             a = a[::-1]
@@ -499,16 +526,16 @@ class Vector(np.ndarray):
         # Can be empty if all of seq are missing values.
         types = util.unique_types(seq)
         if dtype is not None:
-            missing = Vector.fast([], dtype).missing_value
+            na = Vector.fast([], dtype).na_value
         elif len(types) == 1 and types.copy().pop().__module__ == "numpy":
             # If we have a regular Python list of NumPy scalars,
             # infer type. This should be rare, but can happen.
             dtype = types.copy().pop()().dtype
-            missing = Vector.fast([], dtype).missing_value
+            na = Vector.fast([], dtype).na_value
         else:
             # Guess the missing value based on types in seq.
-            missing = cls._std_to_np_missing_value(types)
-        seq = [missing if
+            na = cls._std_to_np_na_value(types)
+        seq = [na if
                x is None or
                (isinstance(x, float) and np.isnan(x))
                else x for x in seq]
@@ -526,7 +553,7 @@ class Vector(np.ndarray):
         return np.array(seq, dtype)
 
     @classmethod
-    def _std_to_np_missing_value(cls, types):
+    def _std_to_np_na_value(cls, types):
         if not types:
             return None
         if str in types:
@@ -551,7 +578,7 @@ class Vector(np.ndarray):
         n = min(self.length, n)
         return self[np.arange(self.length - n, self.length)].copy()
 
-    def to_string(self, max_elements=None):
+    def to_string(self, *, max_elements=None):
         """
         Return vector as a string formatted for display.
 
@@ -563,7 +590,7 @@ class Vector(np.ndarray):
             if len(rows[-1]) <= 1:
                 return rows[-1].append(string)
             row = " ".join(rows[-1] + [string])
-            if len(row) < print_width:
+            if util.ulen(row) < print_width:
                 return rows[-1].append(string)
             # Start a new row with padding and string.
             return rows.append([" ", string])
@@ -574,14 +601,13 @@ class Vector(np.ndarray):
             add_string_element(string, rows)
         if max_elements < self.length:
             add_string_element("...", rows)
-        add_string_element("]", rows)
+        add_string_element(f"] {self.dtype}", rows)
         if len(rows) == 1:
             # Drop padding for single-line output.
             rows[0] = [x.strip() for x in rows[0]]
-        rows.append([f"dtype: {self.dtype}"])
         return "\n".join(" ".join(x) for x in rows)
 
-    def to_strings(self, quote=True, pad=False):
+    def to_strings(self, *, quote=True, pad=False, truncate_width=inf):
         """
         Return vector as strings formatted for display.
 
@@ -596,11 +622,20 @@ class Vector(np.ndarray):
         if self.is_float():
             strings = util.format_floats(self)
             return self.__class__.fast(pad(strings), str)
-        if self.is_integer():
+        if self.is_integer() and not self.is_timedelta():
             strings = ["{:d}".format(x) for x in self]
             return self.__class__.fast(pad(strings), str)
+        if self.is_object():
+            strings = [str(x) for x in self]
+            for i in range(len(strings)):
+                if util.ulen(strings[i]) > truncate_width:
+                    strings[i] = strings[i][:(truncate_width-1)] + "…"
+            return self.__class__.fast(pad(strings), str)
         if self.is_string():
             strings = [quote(x) for x in self]
+            for i in range(len(strings)):
+                if util.ulen(strings[i]) > truncate_width:
+                    strings[i] = strings[i][:(truncate_width-1)] + "…"
             return self.__class__.fast(pad(strings), str)
         strings = [str(x) for x in self]
         return self.__class__.fast(pad(strings), str)
@@ -611,7 +646,7 @@ class Vector(np.ndarray):
 
         Missing values are replaced with ``None``.
         """
-        return np.where(self.is_missing(), None, self).tolist()
+        return np.where(self.is_na(), None, self).tolist()
 
     def unique(self):
         """
